@@ -97,11 +97,15 @@ impl CrateName {
     pub fn normalize_dashes(name: &str) -> CrateName {
         Self(SmolStr::new(name.replace('-', "_")))
     }
+
+    pub fn as_smol_str(&self) -> &SmolStr {
+        &self.0
+    }
 }
 
 impl fmt::Display for CrateName {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(f, "{}", self.0)
+        self.0.fmt(f)
     }
 }
 
@@ -112,6 +116,52 @@ impl ops::Deref for CrateName {
     }
 }
 
+/// Origin of the crates. It is used in emitting monikers.
+#[derive(Debug, Clone, PartialEq, Eq, Hash)]
+pub enum CrateOrigin {
+    /// Crates that are from crates.io official registry,
+    CratesIo { repo: Option<String> },
+    /// Crates that are provided by the language, like std, core, proc-macro, ...
+    Lang(LangCrateOrigin),
+}
+
+#[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
+pub enum LangCrateOrigin {
+    Alloc,
+    Core,
+    ProcMacro,
+    Std,
+    Test,
+    Other,
+}
+
+impl From<&str> for LangCrateOrigin {
+    fn from(s: &str) -> Self {
+        match s {
+            "alloc" => LangCrateOrigin::Alloc,
+            "core" => LangCrateOrigin::Core,
+            "proc-macro" => LangCrateOrigin::ProcMacro,
+            "std" => LangCrateOrigin::Std,
+            "test" => LangCrateOrigin::Test,
+            _ => LangCrateOrigin::Other,
+        }
+    }
+}
+
+impl fmt::Display for LangCrateOrigin {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        let text = match self {
+            LangCrateOrigin::Alloc => "alloc",
+            LangCrateOrigin::Core => "core",
+            LangCrateOrigin::ProcMacro => "proc_macro",
+            LangCrateOrigin::Std => "std",
+            LangCrateOrigin::Test => "test",
+            LangCrateOrigin::Other => "other",
+        };
+        f.write_str(text)
+    }
+}
+
 #[derive(Debug, Clone, PartialEq, Eq, Hash)]
 pub struct CrateDisplayName {
     // The name we use to display various paths (with `_`).
@@ -120,6 +170,15 @@ pub struct CrateDisplayName {
     canonical_name: String,
 }
 
+impl CrateDisplayName {
+    pub fn canonical_name(&self) -> &str {
+        &self.canonical_name
+    }
+    pub fn crate_name(&self) -> &CrateName {
+        &self.crate_name
+    }
+}
+
 impl From<CrateName> for CrateDisplayName {
     fn from(crate_name: CrateName) -> CrateDisplayName {
         let canonical_name = crate_name.to_string();
@@ -129,7 +188,7 @@ impl From<CrateName> for CrateDisplayName {
 
 impl fmt::Display for CrateDisplayName {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(f, "{}", self.crate_name)
+        self.crate_name.fmt(f)
     }
 }
 
@@ -183,6 +242,7 @@ pub struct ProcMacro {
 pub struct CrateData {
     pub root_file_id: FileId,
     pub edition: Edition,
+    pub version: Option<String>,
     /// A name used in the package's project declaration: for Cargo projects,
     /// its `[package].name` can be different for other project types or even
     /// absent (a dummy crate for the code snippet, for example).
@@ -195,6 +255,8 @@ pub struct CrateData {
     pub env: Env,
     pub dependencies: Vec<Dependency>,
     pub proc_macro: Vec<ProcMacro>,
+    pub origin: CrateOrigin,
+    pub is_proc_macro: bool,
 }
 
 #[derive(Debug, Clone, Copy, PartialEq, Eq, PartialOrd, Ord, Hash)]
@@ -241,20 +303,26 @@ impl CrateGraph {
         file_id: FileId,
         edition: Edition,
         display_name: Option<CrateDisplayName>,
+        version: Option<String>,
         cfg_options: CfgOptions,
         potential_cfg_options: CfgOptions,
         env: Env,
         proc_macro: Vec<ProcMacro>,
+        is_proc_macro: bool,
+        origin: CrateOrigin,
     ) -> CrateId {
         let data = CrateData {
             root_file_id: file_id,
             edition,
+            version,
             display_name,
             cfg_options,
             potential_cfg_options,
             env,
             proc_macro,
             dependencies: Vec::new(),
+            origin,
+            is_proc_macro,
         };
         let crate_id = CrateId(self.arena.len() as u32);
         let prev = self.arena.insert(crate_id, data);
@@ -293,7 +361,7 @@ impl CrateGraph {
 
     /// Returns an iterator over all transitive dependencies of the given crate,
     /// including the crate itself.
-    pub fn transitive_deps(&self, of: CrateId) -> impl Iterator<Item = CrateId> + '_ {
+    pub fn transitive_deps(&self, of: CrateId) -> impl Iterator<Item = CrateId> {
         let mut worklist = vec![of];
         let mut deps = FxHashSet::default();
 
@@ -310,7 +378,7 @@ impl CrateGraph {
 
     /// Returns all transitive reverse dependencies of the given crate,
     /// including the crate itself.
-    pub fn transitive_rev_deps(&self, of: CrateId) -> impl Iterator<Item = CrateId> + '_ {
+    pub fn transitive_rev_deps(&self, of: CrateId) -> impl Iterator<Item = CrateId> {
         let mut worklist = vec![of];
         let mut rev_deps = FxHashSet::default();
         rev_deps.insert(of);
@@ -545,6 +613,8 @@ impl fmt::Display for CyclicDependenciesError {
 
 #[cfg(test)]
 mod tests {
+    use crate::CrateOrigin;
+
     use super::{CfgOptions, CrateGraph, CrateName, Dependency, Edition::Edition2018, Env, FileId};
 
     #[test]
@@ -554,28 +624,37 @@ mod tests {
             FileId(1u32),
             Edition2018,
             None,
+            None,
             CfgOptions::default(),
             CfgOptions::default(),
             Env::default(),
             Default::default(),
+            false,
+            CrateOrigin::CratesIo { repo: None },
         );
         let crate2 = graph.add_crate_root(
             FileId(2u32),
             Edition2018,
             None,
-            CfgOptions::default(),
-            CfgOptions::default(),
-            Env::default(),
-            Default::default(),
-        );
-        let crate3 = graph.add_crate_root(
-            FileId(3u32),
-            Edition2018,
             None,
             CfgOptions::default(),
             CfgOptions::default(),
             Env::default(),
             Default::default(),
+            false,
+            CrateOrigin::CratesIo { repo: None },
+        );
+        let crate3 = graph.add_crate_root(
+            FileId(3u32),
+            Edition2018,
+            None,
+            None,
+            CfgOptions::default(),
+            CfgOptions::default(),
+            Env::default(),
+            Default::default(),
+            false,
+            CrateOrigin::CratesIo { repo: None },
         );
         assert!(graph
             .add_dep(crate1, Dependency::new(CrateName::new("crate2").unwrap(), crate2))
@@ -595,19 +674,25 @@ mod tests {
             FileId(1u32),
             Edition2018,
             None,
-            CfgOptions::default(),
-            CfgOptions::default(),
-            Env::default(),
-            Default::default(),
-        );
-        let crate2 = graph.add_crate_root(
-            FileId(2u32),
-            Edition2018,
             None,
             CfgOptions::default(),
             CfgOptions::default(),
             Env::default(),
             Default::default(),
+            false,
+            CrateOrigin::CratesIo { repo: None },
+        );
+        let crate2 = graph.add_crate_root(
+            FileId(2u32),
+            Edition2018,
+            None,
+            None,
+            CfgOptions::default(),
+            CfgOptions::default(),
+            Env::default(),
+            Default::default(),
+            false,
+            CrateOrigin::CratesIo { repo: None },
         );
         assert!(graph
             .add_dep(crate1, Dependency::new(CrateName::new("crate2").unwrap(), crate2))
@@ -624,28 +709,37 @@ mod tests {
             FileId(1u32),
             Edition2018,
             None,
+            None,
             CfgOptions::default(),
             CfgOptions::default(),
             Env::default(),
             Default::default(),
+            false,
+            CrateOrigin::CratesIo { repo: None },
         );
         let crate2 = graph.add_crate_root(
             FileId(2u32),
             Edition2018,
             None,
-            CfgOptions::default(),
-            CfgOptions::default(),
-            Env::default(),
-            Default::default(),
-        );
-        let crate3 = graph.add_crate_root(
-            FileId(3u32),
-            Edition2018,
             None,
             CfgOptions::default(),
             CfgOptions::default(),
             Env::default(),
             Default::default(),
+            false,
+            CrateOrigin::CratesIo { repo: None },
+        );
+        let crate3 = graph.add_crate_root(
+            FileId(3u32),
+            Edition2018,
+            None,
+            None,
+            CfgOptions::default(),
+            CfgOptions::default(),
+            Env::default(),
+            Default::default(),
+            false,
+            CrateOrigin::CratesIo { repo: None },
         );
         assert!(graph
             .add_dep(crate1, Dependency::new(CrateName::new("crate2").unwrap(), crate2))
@@ -662,19 +756,25 @@ mod tests {
             FileId(1u32),
             Edition2018,
             None,
-            CfgOptions::default(),
-            CfgOptions::default(),
-            Env::default(),
-            Default::default(),
-        );
-        let crate2 = graph.add_crate_root(
-            FileId(2u32),
-            Edition2018,
             None,
             CfgOptions::default(),
             CfgOptions::default(),
             Env::default(),
             Default::default(),
+            false,
+            CrateOrigin::CratesIo { repo: None },
+        );
+        let crate2 = graph.add_crate_root(
+            FileId(2u32),
+            Edition2018,
+            None,
+            None,
+            CfgOptions::default(),
+            CfgOptions::default(),
+            Env::default(),
+            Default::default(),
+            false,
+            CrateOrigin::CratesIo { repo: None },
         );
         assert!(graph
             .add_dep(
